//
// crt0.asm
//
// This is the startup code to get a supervisor program going.
//
// For BF533 and BF537 only
// 2004, 2005 Martin Strubel <hackfin@section5.ch>
//
// 2008,  pumpkin
//
//

.text;

#include <defBF537.h>
#include "config.h"


////////////////////////////////////////////////////////////////////////////
// core clock dividers -- DO NOT CHANGE!
#define CCLK_1 0x00
#define CCLK_2 0x10
#define CCLK_4 0x20
#define CCLK_8 0x30

// little macro trick to resolve macros before concatenating:
#define _GET_CCLK(x) CCLK_##x
#define GET_CCLK(x) _GET_CCLK(x)

// Short bootstrap

.global start
start:

	sp.h = 0xFFB0;		//Set up supervisor stack in scratch pad
	sp.l = 0x0400;
	fp = sp;


////////////////////////////////////////////////////////////////////////////
// PLL and clock setups
//
//



setupPLL:
	// we have to enter the idle state after changes applied to the
	// VCO clock, because the PLL needs to lock in on the new clocks.


	p0.l = LO(PLL_CTL);
	p0.h = HI(PLL_CTL);
	r1 = w[p0](z);
	r2 = r1;  
	r0 = 0(z);
		
	r0.l = ~(0x3f << 9);
	r1 = r1 & r0;
	r0.l = ((VCO_MULTIPLIER & 0x3f) << 9);
	r1 = r1 | r0;


#ifdef BOARD_EZKIT_BF561
 	p1.l = LO(SICA_IWR0);  // enable PLL Wakeup Interrupt
	p1.h = HI(SICA_IWR0);
#else
 	p1.l = LO(SIC_IWR);  // enable PLL Wakeup Interrupt
	p1.h = HI(SIC_IWR);
#endif

	r0 = [p1];			
	bitset(r0,0);	  
	[p1] = r0;
	
 	w[p0] = r1;          // Apply PLL_CTL changes.
	ssync;
 	
	cli r0;
 	idle;	// wait for Loop_count expired wake up
	sti r0;

	// now, set clock dividers:
	p0.l = LO(PLL_DIV);
	p0.h = HI(PLL_DIV);


	// SCLK = VCOCLK / SCLK_DIVIDER
	r0.l = (GET_CCLK(CCLK_DIVIDER) | (SCLK_DIVIDER & 0x000f));


	w[p0] = r0; // set Core and system clock dividers


	// not needed in reset routine: sti r1;

////////////////////////////////////////////////////////////////////////////
// Install interrupt handlers.
//
	p0.l = LO(EVT2);
	p0.h = HI(EVT2);

	r0.l = _Interrupt_NMI;
	r0.h = _Interrupt_NMI;
	[p0++] = r0;

	r0.l = _Exception_EVX;
	r0.h = _Exception_EVX;
	[p0++] = r0;
	
	[p0++] = r0;	// Event 4 isn't used.

	r0.l = _Interrupt_IVHW;
	r0.h = _Interrupt_IVHW;
	[p0++] = r0;
	
	r0.l = _Interrupt_IVTMR;
	r0.h = _Interrupt_IVTMR;
	[p0++] = r0;
	
	r0.l = _Interrupt_IVG7;
	r0.h = _Interrupt_IVG7;
	[p0++] = r0;
	
	r0.l = _Interrupt_IVG8;
	r0.h = _Interrupt_IVG8;
	[p0++] = r0;

	r0.l = _Interrupt_IVG9;
	r0.h = _Interrupt_IVG9;
	[p0++] = r0;

	r0.l = _Interrupt_IVG10;
	r0.h = _Interrupt_IVG10;
	[p0++] = r0;

	r0.l = _Interrupt_IVG11;
	r0.h = _Interrupt_IVG11;
	[p0++] = r0;

	r0.l = _Interrupt_IVG12;
	r0.h = _Interrupt_IVG12;
	[p0++] = r0;
	
	r0.l = _Interrupt_IVG13;
	r0.h = _Interrupt_IVG13;
	[p0++] = r0;

	r0.l = _Interrupt_IVG14;
	r0.h = _Interrupt_IVG14;
	[p0++] = r0;

	r0.l = _Interrupt_IVG15;
	r0.h = _Interrupt_IVG15;
	[p0++] = r0;


	

	// we want to run our program in supervisor mode,
	// therefore we need a few tricks:


	//  Enable Interrupt 15 
	p0.l = LO(EVT15);
	p0.h = HI(EVT15);
	r0.l = call_main;  // install isr 15 as caller to main
	r0.h = call_main;
	[p0] = r0;

	r0 = 0x8000(z);    // enable irq 15 only
	sti r0;            // set mask
	raise 15;          // raise sw interrupt
	
	p0.l = wait;
	p0.h = wait;

	reti = p0;
	rti;               // return from reset

wait:
	jump wait;         // wait until irq 15 is being serviced.


call_main:
	[--sp] = reti;  // pushing RETI allows interrupts to occur inside all main routines

	p0.l = _main;
	p0.h = _main;

	r0.l = end;
	r0.h = end;

	rets = r0;      // return address for main()'s RTS

	jump (p0);

end:
	idle;
	jump end;


.global idle_loop
idle_loop:
	idle;
	ssync;
	jump idle_loop;


start.end:

	
////////////////////////////////////////////////////////////////////////////
// we need _atexit if we don't use a libc..

#ifndef USE_LIBC

.global _atexit;
_atexit:
	rts;

#endif
 
